# Generated by Django 5.0.9 on 2024-11-06 13:31

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0009_remove_entry_external_identfier_and_more"),
        ("wbcommission", "0006_commissionrule_consider_zero_percent_for_exclusion"),
        ("wbcrm", "0013_account_action_plan_account_relationship_status_and_more"),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name="commission",
            name="unique_crm_recipient_account",
        ),
        migrations.RemoveConstraint(
            model_name="commission",
            name="unique_portfolio_role_recipient_account",
        ),
        migrations.RemoveConstraint(
            model_name="commission",
            name="unique_account_role_type_recipient_account",
        ),
        migrations.AddConstraint(
            model_name="commission",
            constraint=models.UniqueConstraint(
                condition=models.Q(("crm_recipient__isnull", False)),
                fields=("commission_type", "account", "crm_recipient"),
                name="unique_crm_recipient_account",
            ),
        ),
        migrations.AddConstraint(
            model_name="commission",
            constraint=models.UniqueConstraint(
                condition=models.Q(("portfolio_role_recipient__isnull", False)),
                fields=("commission_type", "account", "portfolio_role_recipient"),
                name="unique_portfolio_role_recipient_account",
            ),
        ),
        migrations.AddConstraint(
            model_name="commission",
            constraint=models.UniqueConstraint(
                condition=models.Q(("account_role_type_recipient__isnull", False)),
                fields=("commission_type", "account", "account_role_type_recipient"),
                name="unique_account_role_type_recipient_account",
            ),
        ),
    ]



from typing import Optional

from wbcore.enums import Unit
from wbcore.metadata.configs import display as dp
from wbcore.metadata.configs.display.view_config import DisplayViewConfig


class RebatePandasViewDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="title", label="Title"),
                *[dp.Field(key="rebate_" + k, label=v) for k, v in self.view.rebate_types.items()],
                dp.Field(key="rebate_total", label="Total Rebate"),
            ]
        )


class RebateProductMarginalityDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="title", label="Product", width=Unit.PIXEL(400)),
                dp.Field(
                    label="Management",
                    open_by_default=False,
                    key=None,
                    children=[
                        dp.Field(key="base_management_fees_percent", label="Base Fees", width=Unit.PIXEL(400)),
                        dp.Field(key="management_fees", label="Fees", width=Unit.PIXEL(120)),
                        dp.Field(key="management_rebates", label="Rebates", width=Unit.PIXEL(120)),
                        dp.Field(key="management_marginality", label="Marginality", width=Unit.PIXEL(120)),
                        dp.Field(
                            key="net_management_marginality",
                            label="Net Marginality",
                            width=Unit.PIXEL(150),
                            formatting_rules=[
                                dp.FormattingRule(
                                    style={
                                        "fontWeight": "bold",
                                    },
                                ),
                            ],
                        ),
                    ],
                ),
                dp.Field(
                    label="Performance",
                    open_by_default=False,
                    key=None,
                    children=[
                        dp.Field(key="base_performance_fees_percent", label="Base Fees", width=Unit.PIXEL(120)),
                        dp.Field(key="performance_fees", label="Fees", width=Unit.PIXEL(120)),
                        dp.Field(key="performance_rebates", label="Rebates", width=Unit.PIXEL(120)),
                        dp.Field(key="performance_marginality", label="Marginality", width=Unit.PIXEL(120)),
                        dp.Field(
                            key="net_performance_marginality",
                            label="Net Marginality",
                            width=Unit.PIXEL(150),
                            formatting_rules=[
                                dp.FormattingRule(
                                    style={
                                        "fontWeight": "bold",
                                    },
                                ),
                            ],
                        ),
                    ],
                ),
                dp.Field(
                    label="Total",
                    open_by_default=False,
                    key=None,
                    children=[
                        dp.Field(key="total_fees", label="Fees", width=Unit.PIXEL(120)),
                        dp.Field(key="total_rebates", label="Rebates", width=Unit.PIXEL(120)),
                        dp.Field(key="total_marginality_percent", label="Marginality", width=Unit.PIXEL(120)),
                        dp.Field(
                            key="total_fees_usd",
                            label="Fees ($)",
                            width=Unit.PIXEL(120),
                            formatting_rules=[
                                dp.FormattingRule(
                                    style={
                                        "fontWeight": "bold",
                                    },
                                ),
                            ],
                        ),
                        dp.Field(
                            key="total_rebates_usd",
                            label="Rebates ($)",
                            width=Unit.PIXEL(120),
                            formatting_rules=[
                                dp.FormattingRule(
                                    style={
                                        "fontWeight": "bold",
                                    },
                                ),
                            ],
                        ),
                        dp.Field(
                            key="total_marginality_usd",
                            label="Net Marginality ($)",
                            width=Unit.PIXEL(120),
                            formatting_rules=[
                                dp.FormattingRule(
                                    style={
                                        "fontWeight": "bold",
                                    },
                                ),
                            ],
                        ),
                    ],
                ),
            ]
        )



from wbcore.menus import ItemPermission, MenuItem
from wbportfolio.permissions import is_manager

REBATE_MENUITEM = MenuItem(
    label="Rebates",
    endpoint="wbcommission:rebatetable-list",
    permission=ItemPermission(permissions=["wbcommission.view_rebate"]),
)
REBATE_MARGINALITY_MENUITEM = MenuItem(
    label="Marginality",
    endpoint="wbcommission:rebatemarginalitytable-list",
    permission=ItemPermission(permissions=["wbcommission.view_rebate"], method=is_manager),
)



from django.utils.translation import gettext as _
from rest_framework.reverse import reverse
from wbcore import serializers as wb_serializers
from wbcore.contrib.icons import WBIcon
from wbcore.enums import RequestType
from wbcore.metadata.configs import buttons as bt
from wbcore.metadata.configs.buttons.view_config import ButtonViewConfig
from wbcore.metadata.configs.display.instance_display.shortcuts import (
    create_simple_display,
)
from wbcore.utils.date import current_quarter_date_start
from wbcrm.models.accounts import Account
from wbcrm.serializers.accounts import AccountRepresentationSerializer


class RebateTableButtonConfig(ButtonViewConfig):
    def get_custom_buttons(self):
        class RecomputeRebateSerializer(wb_serializers.Serializer):
            start_date = wb_serializers.DateField(label=_("Start"), default=current_quarter_date_start)
            only_accounts = wb_serializers.PrimaryKeyRelatedField(queryset=Account.objects.filter(level=0))
            _only_accounts = AccountRepresentationSerializer(
                source="only_accounts", filter_params={"level": 0}, many=True
            )
            prune_existing = wb_serializers.BooleanField(
                default=False,
                help_text="If true, existing rebate for the specified accounts and start date will be deleted before regenerated. Usually necessary if commission rule were lowered or claim removed from account",
            )

        if self.request.user.has_perm("wbcommission.administrate_commission"):
            return {
                bt.ActionButton(
                    method=RequestType.PATCH,
                    identifiers=("wbcommission:rebates",),
                    action_label="Trigger Rebate computation",
                    endpoint=reverse("wbcommission:rebate-recompute", args=[], request=self.request),
                    description_fields="<p>Trigger Rebate computation</p>",
                    serializer=RecomputeRebateSerializer,
                    icon=WBIcon.REGENERATE.icon,
                    title="Trigger Rebate computation",
                    label="Trigger Rebate computation",
                    instance_display=create_simple_display(
                        [["start_date", "prune_existing"], ["only_accounts", "only_accounts"]]
                    ),
                ),
            }
        return {}



from datetime import date

import factory
from psycopg.types.range import DateRange
from wbportfolio.models.roles import PortfolioRole

from wbcommission.models.commission import (
    Commission,
    CommissionExclusionRule,
    CommissionRole,
    CommissionType,
)


class CommissionTypeFactory(factory.django.DjangoModelFactory):
    name = "MANAGEMENT"
    key = factory.LazyAttribute(lambda x: x.name.lower())

    class Meta:
        model = CommissionType
        django_get_or_create = ["key"]


class CommissionFactory(factory.django.DjangoModelFactory):
    account = factory.SubFactory("wbcrm.factories.AccountFactory")
    crm_recipient = factory.SubFactory("wbcore.contrib.directory.factories.entries.EntryFactory")

    portfolio_role_recipient = None
    account_role_type_recipient = None
    order = 0
    commission_type = factory.SubFactory(CommissionTypeFactory)
    net_commission = True
    is_hidden = False

    class Meta:
        model = Commission

    @factory.post_generation
    def rule_timespan(self, create, extracted, **kwargs):
        if not create:
            return
        if extracted:
            v = self.rules.first()
            v.timespan = extracted
            v.save()

    @factory.post_generation
    def rule_aum(self, create, extracted, **kwargs):
        if not create:
            return
        if extracted:
            v = self.rules.first()
            v.assets_under_management_range = extracted
            v.save()

    @factory.post_generation
    def rule_percent(self, create, extracted, **kwargs):
        if not create:
            return
        if extracted:
            v = self.rules.first()
            v.percent = extracted
            v.save()


class PortfolioRoleCommissionFactory(CommissionFactory):
    crm_recipient = None
    account_role_type_recipient = None
    portfolio_role_recipient = factory.Iterator([role_choice[0] for role_choice in PortfolioRole.RoleType.choices])

    class Meta:
        model = Commission


class AccountTypeRoleCommissionFactory(CommissionFactory):
    crm_recipient = None
    portfolio_role_recipient = None
    account_role_type_recipient = factory.SubFactory("wbcrm.factories.AccountRoleTypeFactory")

    class Meta:
        model = Commission


class CommissionRoleFactory(factory.django.DjangoModelFactory):
    commission = factory.SubFactory(CommissionFactory)
    person = factory.SubFactory("wbcore.contrib.directory.factories.entries.PersonFactory")

    class Meta:
        model = CommissionRole


class CommissionExclusionRuleFactory(factory.django.DjangoModelFactory):
    product = factory.SubFactory("wbportfolio.factories.products.ProductFactory")
    commission_type = factory.SubFactory(CommissionTypeFactory)
    overriding_percent = factory.Faker("pydecimal", min_value=0, max_value=1, right_digits=2)
    overriding_net_or_gross_commission = "DEFAULT"
    account_role_type = None
    timespan = DateRange(date.min, date.max)

    class Meta:
        model = CommissionExclusionRule



# Generated by Django 5.0.9 on 2024-11-11 11:59

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbcommission", "0007_remove_commission_unique_crm_recipient_account_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="commission",
            options={
                "ordering": ("order",),
                "permissions": (("administrate_commission", "Can administrate Commission"),),
                "verbose_name": "Commission",
                "verbose_name_plural": "Commissions",
            },
        ),
        migrations.AlterField(
            model_name="commission",
            name="order",
            field=models.PositiveIntegerField(db_index=True, editable=False, verbose_name="order"),
        ),
    ]
